import logging
import os
import zipfile

import wget
from django.conf import settings
from django.core.management.base import BaseCommand

from osm import tasks

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "OSM countries poligons import"
    GDAL_DIR = os.path.join(settings.STATIC_ROOT, "gdal")
    URL_COUNTRIES = "https://thematicmapping.org/downloads/TM_WORLD_BORDERS-0.3.zip"
    OSM_DIR = os.path.join(GDAL_DIR, "countries")

    def handle(self, *args, **options):
        if not os.path.isdir(self.OSM_DIR):
            os.makedirs(self.OSM_DIR)

        file_name = self.URL_COUNTRIES.split("/")[-1]
        self.downloader(self.URL_COUNTRIES, self.OSM_DIR)
        with zipfile.ZipFile(os.path.join(self.OSM_DIR, file_name)) as zip_file:
            zip_file.extractall(self.OSM_DIR)
        tasks.osm_country_import.delay()

    def downloader(self, url, path):
        self.stdout.write(f"Downloading file: {self.URL_COUNTRIES}")
        wget.download(url, path)
        self.stdout.write(self.style.SUCCESS("\tSuccess"))



# Generated by Django 3.2.5 on 2021-08-02 05:23

import django.contrib.gis.db.models.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Building',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Name')),
                ('type', models.CharField(blank=True, max_length=20, null=True, verbose_name='Object Type')),
                ('geom', django.contrib.gis.db.models.fields.PolygonField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Building',
                'verbose_name_plural': 'Buildings',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Country',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('fips', models.CharField(blank=True, max_length=2, null=True, verbose_name='Fips')),
                ('iso2', models.CharField(max_length=2, verbose_name='iso2')),
                ('iso3', models.CharField(max_length=3, verbose_name='iso3')),
                ('un', models.IntegerField(verbose_name='UN')),
                ('name', models.CharField(db_index=True, max_length=50, unique=True, verbose_name='Name')),
                ('area', models.IntegerField(verbose_name='Area')),
                ('pop2005', models.BigIntegerField(verbose_name='Pop 2005')),
                ('region', models.IntegerField(verbose_name='Region')),
                ('subregion', models.IntegerField(verbose_name='Subregion')),
                ('lon', models.FloatField(verbose_name='Lon')),
                ('lat', models.FloatField(verbose_name='Lat')),
                ('geom', django.contrib.gis.db.models.fields.MultiPolygonField(srid=4326, verbose_name='Geom')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Country',
                'verbose_name_plural': 'Countries',
            },
        ),
        migrations.CreateModel(
            name='Landuse',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Name')),
                ('geom', django.contrib.gis.db.models.fields.MultiPolygonField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Landuse',
                'verbose_name_plural': 'Landuses',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Natural',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Name')),
                ('geom', django.contrib.gis.db.models.fields.PointField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Natural',
                'verbose_name_plural': 'Naturals',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='NaturalA',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Name')),
                ('geom', django.contrib.gis.db.models.fields.MultiPolygonField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Natural A',
                'verbose_name_plural': 'Naturals A',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Place',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('population', models.BigIntegerField()),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Name')),
                ('geom', django.contrib.gis.db.models.fields.MultiPointField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Place',
                'verbose_name_plural': 'Places',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='PlaceA',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('population', models.BigIntegerField()),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Name')),
                ('geom', django.contrib.gis.db.models.fields.MultiPolygonField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Place A',
                'verbose_name_plural': 'Places A',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Pofw',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Name')),
                ('geom', django.contrib.gis.db.models.fields.MultiPointField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Pofw',
                'verbose_name_plural': 'Pofw',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='PofwA',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Name')),
                ('geom', django.contrib.gis.db.models.fields.MultiPolygonField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Pofw A',
                'verbose_name_plural': 'Pofw A',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Pois',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Name')),
                ('geom', django.contrib.gis.db.models.fields.PointField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Pois',
                'verbose_name_plural': 'Poises',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='PoisA',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Name')),
                ('geom', django.contrib.gis.db.models.fields.MultiPolygonField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Pois A',
                'verbose_name_plural': 'Poises A',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='RailWay',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Name')),
                ('layer', models.BigIntegerField(verbose_name='Layer')),
                ('bridge', models.CharField(max_length=1, verbose_name='Bridge')),
                ('tunnel', models.CharField(max_length=1, verbose_name='Tunnel')),
                ('geom', django.contrib.gis.db.models.fields.MultiLineStringField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'RailWay',
                'verbose_name_plural': 'RailWays',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Road',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Object name')),
                ('ref', models.CharField(max_length=20, null=True, verbose_name='Ref')),
                ('oneway', models.CharField(max_length=1, verbose_name='One way')),
                ('maxspeed', models.IntegerField(verbose_name='Max speed')),
                ('layer', models.BigIntegerField(verbose_name='Layer')),
                ('bridge', models.CharField(max_length=1, verbose_name='Bridge')),
                ('tunnel', models.CharField(max_length=1, verbose_name='Tunnel')),
                ('geom', django.contrib.gis.db.models.fields.MultiLineStringField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Road',
                'verbose_name_plural': 'Roads',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Traffic',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Object name')),
                ('geom', django.contrib.gis.db.models.fields.MultiPointField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Traffic',
                'verbose_name_plural': 'Traffics',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='TrafficA',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Object name')),
                ('geom', django.contrib.gis.db.models.fields.MultiPolygonField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Traffic A',
                'verbose_name_plural': 'Traffics A',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Transport',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Object name')),
                ('geom', django.contrib.gis.db.models.fields.MultiPointField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Transport',
                'verbose_name_plural': 'Transports',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='TransportA',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Object name')),
                ('geom', django.contrib.gis.db.models.fields.MultiPolygonField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Transport A',
                'verbose_name_plural': 'Transports A',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='Water',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Object name')),
                ('geom', django.contrib.gis.db.models.fields.MultiPolygonField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Water',
                'verbose_name_plural': 'Waters',
                'ordering': ['-id'],
            },
        ),
        migrations.CreateModel(
            name='WaterWay',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('osm_id', models.CharField(db_index=True, max_length=10, unique=True, verbose_name='OSM ID')),
                ('code', models.IntegerField(verbose_name='Code')),
                ('fclass', models.CharField(max_length=28, verbose_name='Object class')),
                ('width', models.IntegerField()),
                ('name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Object name')),
                ('geom', django.contrib.gis.db.models.fields.MultiLineStringField(srid=4326)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('deleted', models.BooleanField(default=False, verbose_name='Is deleted')),
            ],
            options={
                'verbose_name': 'Water Way',
                'verbose_name_plural': 'WaterWays',
                'ordering': ['-id'],
            },
        ),
    ]



import os

from celery import shared_task
from celery.utils.log import get_task_logger
from django.conf import settings
from django.contrib.gis.utils import LayerMapping

from osm import models

logger = get_task_logger(__name__)


@shared_task(bind=True, ignore_results=True)
def osm_import(self, osm_model, osm_filename, verbose=True):
    osm_model = getattr(models, osm_model)
    lm = LayerMapping(
        osm_model,
        osm_filename,
        osm_model.mapping(),
        transform=True,
        transaction_mode="autocommit",
        unique="osm_id",
    )
    lm.save(strict=True, verbose=verbose)


@shared_task(bind=True, ignore_results=True)
def osm_country_import(self, verbose=True):
    osm_model = models.Country
    osm_filename = os.path.join(
        settings.STATIC_ROOT, "gdal", "countries", "TM_WORLD_BORDERS-0.3.shp"
    )
    lm = LayerMapping(
        osm_model,
        osm_filename,
        osm_model.mapping(),
        transform=True,
        transaction_mode="autocommit",
        unique="name",
    )
    lm.save(strict=True, verbose=verbose)



# '69.279737,41.311273'
from django.contrib.gis.db.models.functions import Distance
from django.contrib.gis.geos import Point
from rest_framework.filters import BaseFilterBackend


class PointDistFilter(BaseFilterBackend):
    def filter_queryset(self, request, queryset, view):
        request_keys = request.GET.keys()
        if "dist" in request_keys and "point" in request_keys:
            dist = float(request.GET["dist"])
            point = request.GET["point"].split(",")
            point = Point(float(point[0]), float(point[1]), srid=4326)
            queryset = (
                queryset.annotate(distance=Distance("geom", point))
                .filter(distance__lte=dist)
                .order_by("distance")
            )
        return queryset


class PointIntersectsFilter(BaseFilterBackend):
    def filter_queryset(self, request, queryset, view):
        if "intersects" in request.GET.keys():
            point = request.GET["intersects"].split(",")
            point = Point(float(point[0]), float(point[1]))
            queryset = queryset.filter(geom__intersects=point)
        return queryset


class PointContainsFilter(BaseFilterBackend):
    def filter_queryset(self, request, queryset, view):
        if "contains" in request.GET.keys():
            point = request.GET["contains"].split(",")
            point = Point(float(point[0]), float(point[1]))
            queryset = queryset.filter(geom__intersects=point)
        return queryset



# -*- coding: utf-8 -*-
from django.contrib.gis.db import models
from django.utils.translation import gettext_lazy as _


class Country(models.Model):
    fips = models.CharField(max_length=2, blank=True, null=True, verbose_name=_("Fips"))
    iso2 = models.CharField(max_length=2, verbose_name=_("iso2"))
    iso3 = models.CharField(max_length=3, verbose_name=_("iso3"))
    un = models.IntegerField(verbose_name=_("UN"))
    name = models.CharField(
        max_length=50, unique=True, db_index=True, verbose_name=_("Name")
    )
    area = models.IntegerField(verbose_name=_("Area"))
    pop2005 = models.BigIntegerField(verbose_name=_("Pop 2005"))
    region = models.IntegerField(verbose_name=_("Region"))
    subregion = models.IntegerField(verbose_name=_("Subregion"))
    lon = models.FloatField(verbose_name=_("Lon"))
    lat = models.FloatField(verbose_name=_("Lat"))
    geom = models.MultiPolygonField(verbose_name=_("Geom"))
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    @staticmethod
    def mapping():
        return {
            "fips": "FIPS",
            "iso2": "ISO2",
            "iso3": "ISO3",
            "un": "UN",
            "name": "NAME",
            "area": "AREA",
            "pop2005": "POP2005",
            "region": "REGION",
            "subregion": "SUBREGION",
            "lon": "LON",
            "lat": "LAT",
            "geom": "MULTIPOLYGON",
        }

    def __str__(self):
        return f"Country({self.name})"

    class Meta:
        verbose_name = _("Country")
        verbose_name_plural = _("Countries")


class Building(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))

    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Name")
    )
    type = models.CharField(
        max_length=20, null=True, blank=True, verbose_name=_("Object Type")
    )
    geom = models.PolygonField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"Building({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "name": "name",
            "type": "type",
            "geom": "POLYGON",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("Building")
        verbose_name_plural = _("Buildings")
        app_label = "osm"


class Landuse(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))
    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Name")
    )
    geom = models.MultiPolygonField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"Landuse({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "name": "name",
            "geom": "MULTIPOLYGON",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("Landuse")
        verbose_name_plural = _("Landuses")
        app_label = "osm"


class Natural(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))
    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Name")
    )
    geom = models.PointField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"Natural({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "name": "name",
            "geom": "POINT",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("Natural")
        verbose_name_plural = _("Naturals")
        app_label = "osm"


class NaturalA(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))
    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Name")
    )
    geom = models.MultiPolygonField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"NaturalA({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "name": "name",
            "geom": "MULTIPOLYGON",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("Natural A")
        verbose_name_plural = _("Naturals A")
        app_label = "osm"


class Place(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))
    population = models.BigIntegerField()
    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Name")
    )
    geom = models.MultiPointField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"Place({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "population": "population",
            "name": "name",
            "geom": "MULTIPOINT",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("Place")
        verbose_name_plural = _("Places")
        app_label = "osm"


class PlaceA(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))
    population = models.BigIntegerField()
    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Name")
    )
    geom = models.MultiPolygonField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"PlaceA({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "population": "population",
            "name": "name",
            "geom": "MULTIPOLYGON",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("Place A")
        verbose_name_plural = _("Places A")
        app_label = "osm"


class Pofw(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))
    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Name")
    )
    geom = models.MultiPointField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"Pofw({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "name": "name",
            "geom": "MULTIPOINT",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("Pofw")
        verbose_name_plural = _("Pofw")
        app_label = "osm"


class PofwA(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))
    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Name")
    )
    geom = models.MultiPolygonField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"PofwA({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "name": "name",
            "geom": "MULTIPOLYGON",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("Pofw A")
        verbose_name_plural = _("Pofw A")
        app_label = "osm"


class Pois(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))
    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Name")
    )
    geom = models.PointField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"Pois({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "name": "name",
            "geom": "POINT",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("Pois")
        verbose_name_plural = _("Poises")
        app_label = "osm"


class PoisA(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))
    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Name")
    )
    geom = models.MultiPolygonField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"PoisA({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "name": "name",
            "geom": "MULTIPOLYGON",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("Pois A")
        verbose_name_plural = _("Poises A")
        app_label = "osm"


class RailWay(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))
    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Name")
    )
    layer = models.BigIntegerField(verbose_name=_("Layer"))
    bridge = models.CharField(max_length=1, verbose_name=_("Bridge"))
    tunnel = models.CharField(max_length=1, verbose_name=_("Tunnel"))
    geom = models.MultiLineStringField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"RailWay({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "name": "name",
            "layer": "layer",
            "bridge": "bridge",
            "tunnel": "tunnel",
            "geom": "MULTILINESTRING",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("RailWay")
        verbose_name_plural = _("RailWays")
        app_label = "osm"


class Road(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))
    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Object name")
    )
    ref = models.CharField(max_length=20, null=True, verbose_name=_("Ref"))
    oneway = models.CharField(max_length=1, verbose_name=_("One way"))
    maxspeed = models.IntegerField(verbose_name=_("Max speed"))
    layer = models.BigIntegerField(verbose_name=_("Layer"))
    bridge = models.CharField(max_length=1, verbose_name=_("Bridge"))
    tunnel = models.CharField(max_length=1, verbose_name=_("Tunnel"))
    geom = models.MultiLineStringField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"Road({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "name": "name",
            "ref": "ref",
            "oneway": "oneway",
            "maxspeed": "maxspeed",
            "layer": "layer",
            "bridge": "bridge",
            "tunnel": "tunnel",
            "geom": "MULTILINESTRING",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("Road")
        verbose_name_plural = _("Roads")
        app_label = "osm"


class Traffic(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))
    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Object name")
    )
    geom = models.MultiPointField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"Traffic({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "name": "name",
            "geom": "MULTIPOINT",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("Traffic")
        verbose_name_plural = _("Traffics")
        app_label = "osm"


class TrafficA(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))
    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Object name")
    )
    geom = models.MultiPolygonField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"TrafficA({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "name": "name",
            "geom": "MULTIPOLYGON",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("Traffic A")
        verbose_name_plural = _("Traffics A")
        app_label = "osm"


class Transport(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))
    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Object name")
    )
    geom = models.MultiPointField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"Transport({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "name": "name",
            "geom": "MULTIPOINT",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("Transport")
        verbose_name_plural = _("Transports")
        app_label = "osm"


class TransportA(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))
    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Object name")
    )
    geom = models.MultiPolygonField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"TransportA({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "name": "name",
            "geom": "MULTIPOLYGON",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("Transport A")
        verbose_name_plural = _("Transports A")
        app_label = "osm"


class Water(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))
    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Object name")
    )
    geom = models.MultiPolygonField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"Water({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "name": "name",
            "geom": "MULTIPOLYGON",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("Water")
        verbose_name_plural = _("Waters")
        app_label = "osm"


class WaterWay(models.Model):
    osm_id = models.CharField(
        max_length=10, unique=True, db_index=True, verbose_name=_("OSM ID")
    )
    code = models.IntegerField(verbose_name=_("Code"))
    fclass = models.CharField(max_length=28, verbose_name=_("Object class"))
    width = models.IntegerField()
    name = models.CharField(
        max_length=100, null=True, blank=True, verbose_name=_("Object name")
    )
    geom = models.MultiLineStringField()
    created = models.DateTimeField(auto_now_add=True, verbose_name=_("Created"))
    updated = models.DateTimeField(auto_now=True, verbose_name=_("Updated"))
    deleted = models.BooleanField(default=False, verbose_name=_("Is deleted"))

    def __str__(self):
        return f"WaterWay({self.osm_id})"

    @staticmethod
    def mapping():
        return {
            "osm_id": "osm_id",
            "code": "code",
            "fclass": "fclass",
            "width": "width",
            "name": "name",
            "geom": "MULTILINESTRING",
        }

    class Meta:
        ordering = ["-id"]
        verbose_name = _("Water Way")
        verbose_name_plural = _("WaterWays")
        app_label = "osm"



from rest_framework_gis.serializers import GeoFeatureModelSerializer

from osm import models


class CountrySerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.Country
        fields = "__all__"
        geo_field = "geom"


class BuildingSerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.Building
        fields = "__all__"
        geo_field = "geom"


class LanduseSerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.Landuse
        fields = "__all__"
        geo_field = "geom"


class NaturalSerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.Natural
        fields = "__all__"
        geo_field = "geom"


class NaturalASerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.NaturalA
        fields = "__all__"
        geo_field = "geom"


class PlaceSerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.Place
        fields = "__all__"
        geo_field = "geom"


class PlaceASerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.PlaceA
        fields = "__all__"
        geo_field = "geom"


class PofwSerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.Pofw
        fields = "__all__"
        geo_field = "geom"


class PofwASerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.PofwA
        fields = "__all__"
        geo_field = "geom"


class PoisSerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.Pois
        fields = "__all__"
        geo_field = "geom"


class PoisASerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.PoisA
        fields = "__all__"
        geo_field = "geom"


class RailWaySerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.RailWay
        fields = "__all__"
        geo_field = "geom"


class RoadSerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.Road
        fields = "__all__"
        geo_field = "geom"


class TrafficSerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.Traffic
        fields = "__all__"
        geo_field = "geom"


class TrafficASerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.TrafficA
        fields = "__all__"
        geo_field = "geom"


class TransportSerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.Transport
        fields = "__all__"
        geo_field = "geom"


class TransportASerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.TransportA
        fields = "__all__"
        geo_field = "geom"


class WaterWaySerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.WaterWay
        fields = "__all__"
        geo_field = "geom"


class WaterSerializer(GeoFeatureModelSerializer):
    class Meta:
        model = models.Water
        fields = "__all__"
        geo_field = "geom"



# -*- coding: utf-8 -*-
from django.contrib.gis import admin

from osm import models


@admin.register(models.Country)
class CountryAdmin(admin.OSMGeoAdmin):
    list_display = [
        "name",
        "fips",
        "iso2",
        "iso3",
        "un",
        "area",
        "pop2005",
        "created",
        "updated",
        "deleted",
    ]
    search_fields = ["name"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.Building)
class BuildingAdmin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.Landuse)
class LanduseAdmin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.Natural)
class NaturalAdmin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.NaturalA)
class NaturalA_Admin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.Place)
class PlaceAdmin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.PlaceA)
class PlaceA_Admin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    list_filter = ["fclass"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.Pofw)
class PofwAdmin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.PofwA)
class PofwA_Admin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.Pois)
class PoisAdmin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.PoisA)
class PoisA_Admin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.RailWay)
class RailWayAdmin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.Road)
class RoadAdmin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.Traffic)
class TrafficAdmin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.TrafficA)
class TrafficA_Admin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.Transport)
class TransportAdmin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.TransportA)
class TransportA_Admin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.Water)
class WaterAdmin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]


@admin.register(models.WaterWay)
class WaterWayAdmin(admin.OSMGeoAdmin):
    list_display = ["osm_id", "name", "code", "fclass", "created", "updated", "deleted"]
    list_filter = ["fclass"]
    search_fields = ["name", "osm_id"]
    date_hierarchy = "updated"
    list_editable = ["deleted"]



# -*- coding: utf-8 -*-
from django.urls import path

from osm.api import views

app_name = "osm"
urlpatterns = [
    path("country/", views.CountryListAPIView.as_view(), name="country-list"),
    path("building/", views.BuildingListAPIView.as_view(), name="building-list"),
    path("landuse/", views.LanduseListAPIView.as_view(), name="landuse-list"),
    path("natural/", views.NaturalListAPIView.as_view(), name="natural-list"),
    path("naturala/", views.NaturalAListAPIView.as_view(), name="naturala-list"),
    path("place/", views.PlaceListAPIView.as_view(), name="place-list"),
    path("placea/", views.PlaceAListAPIView.as_view(), name="placea-list"),
    path("pofw/", views.PofwListAPIView.as_view(), name="pofw-list"),
    path("pofwa/", views.PofwAListAPIView.as_view(), name="pofwa-list"),
    path("pois/", views.PoisListAPIView.as_view(), name="pois-list"),
    path("poisa/", views.PoisAListAPIView.as_view(), name="poisa-list"),
    path("railway/", views.RailWayListAPIView.as_view(), name="railway-list"),
    path("road/", views.RoadListAPIView.as_view(), name="road-list"),
    path("traffic/", views.TrafficListAPIView.as_view(), name="traffic-list"),
    path("traffica/", views.TrafficAListAPIView.as_view(), name="traffica-list"),
    path("transport/", views.TransportListAPIView.as_view(), name="transport-list"),
    path(
        "transporta/",
        views.TransportAListAPIView.as_view(),
        name="transporta-list",
    ),
    path("water/", views.WaterListAPIView.as_view(), name="water-list"),
    path("waterway/", views.WaterWayListAPIView.as_view(), name="waterway-list"),
]



import logging
import os
import zipfile

import wget
from django.conf import settings
from django.core.management.base import BaseCommand

from osm import tasks

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Import OSM poligons"
    GDAL_DIR = os.path.join(settings.STATIC_ROOT, "gdal")
    URL_UZBEKISTAN = "https://download.geofabrik.de/asia/uzbekistan-latest-free.shp.zip"
    OSM_DIR = os.path.join(GDAL_DIR, "osm")

    def add_arguments(self, parser):
        parser.add_argument(
            "--Uzbekistan",
            action="store_true",
            default=False,
            help="Download and loading from OSM Server for Tashkent city",
        )
        parser.add_argument(
            "--URL",
            action="store",
            type=str,
            default=False,
            help="Download and loading from URL",
        )

    def handle(self, *args, **options):
        if not os.path.isdir(self.OSM_DIR):
            os.makedirs(self.OSM_DIR)
        if options["Uzbekistan"]:
            self.load_osm(self.URL_UZBEKISTAN)
        else:
            self.load_osm(options["URL"])

    def load_osm(self, url: str):
        file_name = url.split("/")[-1]
        self.downloader(url, self.OSM_DIR)
        with zipfile.ZipFile(os.path.join(self.OSM_DIR, file_name)) as zip_file:
            zip_file.extractall(self.OSM_DIR)
        tasks.osm_import.delay(
            "Building", os.path.join(self.OSM_DIR, "gis_osm_buildings_a_free_1.shp")
        )
        tasks.osm_import.delay(
            "Landuse", os.path.join(self.OSM_DIR, "gis_osm_landuse_a_free_1.shp")
        )
        tasks.osm_import.delay(
            "Natural", os.path.join(self.OSM_DIR, "gis_osm_natural_free_1.shp")
        )
        tasks.osm_import.delay(
            "NaturalA", os.path.join(self.OSM_DIR, "gis_osm_natural_a_free_1.shp")
        )
        tasks.osm_import.delay(
            "Place", os.path.join(self.OSM_DIR, "gis_osm_places_free_1.shp")
        )
        tasks.osm_import.delay(
            "PlaceA", os.path.join(self.OSM_DIR, "gis_osm_places_a_free_1.shp")
        )
        tasks.osm_import.delay(
            "Pofw", os.path.join(self.OSM_DIR, "gis_osm_pofw_free_1.shp")
        )
        tasks.osm_import.delay(
            "PofwA", os.path.join(self.OSM_DIR, "gis_osm_pofw_a_free_1.shp")
        )
        tasks.osm_import.delay(
            "Pois", os.path.join(self.OSM_DIR, "gis_osm_pois_free_1.shp")
        )
        tasks.osm_import.delay(
            "PoisA", os.path.join(self.OSM_DIR, "gis_osm_pois_a_free_1.shp")
        )
        tasks.osm_import.delay(
            "RailWay", os.path.join(self.OSM_DIR, "gis_osm_railways_free_1.shp")
        )
        tasks.osm_import.delay(
            "Road", os.path.join(self.OSM_DIR, "gis_osm_roads_free_1.shp")
        )
        tasks.osm_import.delay(
            "Traffic", os.path.join(self.OSM_DIR, "gis_osm_traffic_free_1.shp")
        )
        tasks.osm_import.delay(
            "TrafficA", os.path.join(self.OSM_DIR, "gis_osm_traffic_a_free_1.shp")
        )
        tasks.osm_import.delay(
            "Transport", os.path.join(self.OSM_DIR, "gis_osm_transport_free_1.shp")
        )
        tasks.osm_import.delay(
            "TransportA", os.path.join(self.OSM_DIR, "gis_osm_transport_a_free_1.shp")
        )
        tasks.osm_import.delay(
            "Water", os.path.join(self.OSM_DIR, "gis_osm_water_a_free_1.shp")
        )
        tasks.osm_import.delay(
            "WaterWay", os.path.join(self.OSM_DIR, "gis_osm_waterways_free_1.shp")
        )

    def downloader(self, url: str, path: str) -> None:
        self.stdout.write(f"Downloading file: {url}")
        wget.download(url, path)
        self.stdout.write(self.style.SUCCESS("\tSuccess"))

    def bunzip2(self, file: str) -> None:
        pass



from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import generics
from rest_framework_gis.pagination import GeoJsonPagination

from osm import models
from osm.api import filters, serializers


class CountryListAPIView(generics.ListAPIView):
    queryset = models.Country.objects.filter(deleted=False)
    serializer_class = serializers.CountrySerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
    ]
    pagination_class = GeoJsonPagination


class BuildingListAPIView(generics.ListAPIView):
    queryset = models.Building.objects.filter(deleted=False)
    serializer_class = serializers.BuildingSerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination


class LanduseListAPIView(generics.ListAPIView):
    queryset = models.Landuse.objects.filter(deleted=False)
    serializer_class = serializers.LanduseSerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination


class NaturalListAPIView(generics.ListAPIView):
    queryset = models.Natural.objects.filter(deleted=False)
    serializer_class = serializers.NaturalSerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination


class NaturalAListAPIView(generics.ListAPIView):
    queryset = models.NaturalA.objects.filter(deleted=False)
    serializer_class = serializers.NaturalASerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination


class PlaceListAPIView(generics.ListAPIView):
    queryset = models.Place.objects.filter(deleted=False)
    serializer_class = serializers.PlaceSerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination


class PlaceAListAPIView(generics.ListAPIView):
    queryset = models.PlaceA.objects.filter(deleted=False)
    serializer_class = serializers.PlaceASerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination


class PofwListAPIView(generics.ListAPIView):
    queryset = models.Pofw.objects.filter(deleted=False)
    serializer_class = serializers.PofwSerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination


class PofwAListAPIView(generics.ListAPIView):
    queryset = models.PofwA.objects.filter(deleted=False)
    serializer_class = serializers.PofwASerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination


class PoisListAPIView(generics.ListAPIView):
    queryset = models.Pois.objects.filter(deleted=False)
    serializer_class = serializers.PoisSerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination


class PoisAListAPIView(generics.ListAPIView):
    queryset = models.PoisA.objects.filter(deleted=False)
    serializer_class = serializers.PoisASerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination


class RailWayListAPIView(generics.ListAPIView):
    queryset = models.RailWay.objects.filter(deleted=False)
    serializer_class = serializers.RailWaySerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination


class RoadListAPIView(generics.ListAPIView):
    queryset = models.Road.objects.filter(deleted=False)
    serializer_class = serializers.RoadSerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination


class TrafficListAPIView(generics.ListAPIView):
    queryset = models.Traffic.objects.filter(deleted=False)
    serializer_class = serializers.TrafficSerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination


class TrafficAListAPIView(generics.ListAPIView):
    queryset = models.TrafficA.objects.filter(deleted=False)
    serializer_class = serializers.TrafficASerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination


class TransportListAPIView(generics.ListAPIView):
    queryset = models.Transport.objects.filter(deleted=False)
    serializer_class = serializers.TransportSerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination


class TransportAListAPIView(generics.ListAPIView):
    queryset = models.TransportA.objects.filter(deleted=False)
    serializer_class = serializers.TrafficASerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination


class WaterListAPIView(generics.ListAPIView):
    queryset = models.Water.objects.filter(deleted=False)
    serializer_class = serializers.WaterSerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination


class WaterWayListAPIView(generics.ListAPIView):
    queryset = models.WaterWay.objects.filter(deleted=False)
    serializer_class = serializers.WaterWaySerializer
    filter_backends = [
        filters.PointDistFilter,
        filters.PointIntersectsFilter,
        filters.PointContainsFilter,
        DjangoFilterBackend,
    ]
    filterset_fields = ("fclass",)
    pagination_class = GeoJsonPagination



""" Route docstrings go here. """
import random

from django.conf import settings


class Default:
    """
    This setting maps database aliases, which are a way to refer to a specific database throughout Django,
    to a dictionary of settings for that specific connection. The settings in the inner dictionaries are
    described fully in the DATABASES documentation.
    """

    def db_for_read(self, model, **hints):
        """Suggest the database that should be used for read operations for objects of type model."""
        if model._meta.app_label in ["osm"]:
            return random.choice(settings.OSM_REPLICS)
        return None

    def db_for_write(self, model, **hints):
        """Suggest the database that should be used for writes of objects of type Model."""
        if model._meta.app_label in ["osm"]:
            return "osm"
        return None

    def allow_relation(self, obj1, obj2, **hints):
        """
        Return True if a relation between obj1 and obj2 should be allowed, False if the relation should be prevented,
        or None if the router has no opinion. This is purely a validation operation, used by foreign key and
        many to many operations to determine if a relation should be allowed between two objects.
        """
        if obj1._meta.app_label in ["osm"] or obj2._meta.app_label == ["osm"]:
            return True
        return None

    def allow_migrate(self, db, app_label, model=None, **hints):
        """
        Determine if the migration operation is allowed to run on the database with alias db. Return True
        if the operation should run, False if it shouldn’t run, or None if the router has no opinion.
        """
        if app_label in ["osm"]:
            return db == "osm"
        return None
